#include "stdafx.h"
#include <iostream>
#include <string>
#include <algorithm>
#include <windows.h>

using namespace std;

#include "CppUnitTest.h"
#include "ISymbolTable.h"
#include "BinaryTree.h"
#include "RedBlackSymbolTable.h"
#include "SplaySymbolTable.h"
#include "VectorSymbolTable.h"
#include "MapSymbolTable.h"
#include "SymbolTableFactory.h"

using namespace cs242;

using namespace Microsoft::VisualStudio::CppUnitTestFramework;

namespace testSedgewickTree
{
	string* generateAll(int nChars);

	TEST_CLASS(perfTest)
	{
	public:

		TEST_METHOD(generateRandomString)
		{
			const int N_TOP_PLAYERS = 10;
			srand(2342525);
			string topPlayers[N_TOP_PLAYERS] = { "Carlsen", "Maxime", "Kramnik", "Aronian",
				"Caruana", "Mamedyarov", "Anand", "So", "Grischuk", "Nakamura" };

			random_shuffle(topPlayers, topPlayers + N_TOP_PLAYERS);

			string result[N_TOP_PLAYERS] = { "Caruana", "Mamedyarov", "Anand",
				"Nakamura", "Carlsen", "So", "Grischuk", "Maxime", "Aronian",
				"Kramnik" };

			for (int i = 0; i < N_TOP_PLAYERS; i++)
			{
				Logger::WriteMessage(topPlayers[i].c_str());
				Assert::AreEqual(result[i], topPlayers[i]);
			}
		}

		TEST_METHOD(generateLittleAZ)
		{
			const int N_CHARS = 2;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);

			Assert::AreEqual(string("aa"), allNChars[0]);
			Assert::AreEqual(string("ab"), allNChars[1]);
			Assert::AreEqual(string("az"), allNChars[BASE - 1]);
			Assert::AreEqual(string("ba"), allNChars[BASE]);
			Assert::AreEqual(string("zz"), allNChars[BASE*BASE - 1]);

			delete[] allNChars;
		}

		TEST_METHOD(generateMediumAZ)
		{
			const int N_CHARS = 3;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);

			Assert::AreEqual(string("aaa"), allNChars[0]);
			Assert::AreEqual(string("aab"), allNChars[1]);
			Assert::AreEqual(string("aaz"), allNChars[BASE - 1]);
			Assert::AreEqual(string("aba"), allNChars[BASE]);
			Assert::AreEqual(string("zzy"), allNChars[BASE * BASE * BASE - 2]);
			Assert::AreEqual(string("zzz"), allNChars[BASE * BASE * BASE - 1]);

			delete[] allNChars;
		}

		TEST_METHOD(generateLargeAZ)
		{
			const int N_CHARS = 4;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);

			Assert::AreEqual(string("aaaa"), allNChars[0]);
			Assert::AreEqual(string("aaab"), allNChars[1]);
			Assert::AreEqual(string("aaaz"), allNChars[BASE - 1]);
			Assert::AreEqual(string("aaba"), allNChars[BASE]);
			Assert::AreEqual(string("zzzy"), allNChars[BASE* BASE * BASE * BASE - 2]);
			Assert::AreEqual(string("zzzz"), allNChars[BASE* BASE * BASE * BASE - 1]);

			delete[] allNChars;
		}

		TEST_METHOD(timeAllRandomInsert)
		{
			const int N_CHARS = 2;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);
			srand(2342523);

			random_shuffle(allNChars, allNChars + length);

			LARGE_INTEGER startTime;
			LARGE_INTEGER endTime;
			const int N_TYPES = 5;
			Logger::WriteMessage("timeAllRandomInsert");
			for (int i = 0; i < N_TYPES; ++i)
			{
				ISymbolTable<int>* symbolTable =
					SymbolTableFactory<int>::create((SymbolTableFactory<int>::TableType) i);

				QueryPerformanceCounter(&startTime);
				for (int i = 0; i < length; i++)
				{
					symbolTable->insert(allNChars[i], i);
				}
				QueryPerformanceCounter(&endTime);

				stringstream timeString;
				timeString << endTime.QuadPart - startTime.QuadPart;
				Logger::WriteMessage(timeString.str().c_str());

				delete symbolTable;
			}
			delete[] allNChars;
		}

		TEST_METHOD(timeAllAlphaInsert)
		{
			const int N_CHARS = 2;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);
			srand(2342523);

			LARGE_INTEGER startTime;
			LARGE_INTEGER endTime;
			const int N_TYPES = 5;
			Logger::WriteMessage("timeAllAlphaInsert");
			for (int i = 0; i < N_TYPES; ++i)
			{
				ISymbolTable<int>* symbolTable =
					SymbolTableFactory<int>::create((SymbolTableFactory<int>::TableType) i);

				QueryPerformanceCounter(&startTime);
				for (int i = 0; i < length; i++)
				{
					symbolTable->insert(allNChars[i], i);
				}
				QueryPerformanceCounter(&endTime);

				stringstream timeString;
				timeString << endTime.QuadPart - startTime.QuadPart;
				Logger::WriteMessage(timeString.str().c_str());

				delete symbolTable;
			}
			delete[] allNChars;
		}

		TEST_METHOD(timeAllAlphLookup)
		{
			const int N_CHARS = 2;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);
			srand(2342523);

			random_shuffle(allNChars, allNChars + length);

			LARGE_INTEGER startTime;
			LARGE_INTEGER endTime;
			const int N_TYPES = 5;
			Logger::WriteMessage("timeAllAlphaLookup");
			for (int i = 0; i < N_TYPES; ++i)
			{
				ISymbolTable<int>* symbolTable =
					SymbolTableFactory<int>::create((SymbolTableFactory<int>::TableType) i);

				for (int i = 0; i < length; i++)
				{
					symbolTable->insert(allNChars[i], i);
				}

				int found;
				QueryPerformanceCounter(&startTime);
				for (int i = 0; i < length; i++)
				{
					symbolTable->lookup(allNChars[i], found);
				}
				QueryPerformanceCounter(&endTime);

				stringstream timeString;
				timeString << endTime.QuadPart - startTime.QuadPart;
				Logger::WriteMessage(timeString.str().c_str());

				delete symbolTable;
			}
			delete[] allNChars;
		}

		TEST_METHOD(timeAllRandomLookup)
		{
			const int N_CHARS = 2;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);
			srand(2342523);

			random_shuffle(allNChars, allNChars + length);

			LARGE_INTEGER startTime;
			LARGE_INTEGER endTime;
			const int N_TYPES = 5;

			Logger::WriteMessage("timeAllRandomLookup");
			for (int i = 0; i < N_TYPES; ++i)
			{
				ISymbolTable<int>* symbolTable =
					SymbolTableFactory<int>::create((SymbolTableFactory<int>::TableType) i);

				for (int i = 0; i < length; i++)
				{
					symbolTable->insert(allNChars[i], i);
				}

				random_shuffle(allNChars, allNChars + length);

				int found;
				QueryPerformanceCounter(&startTime);
				for (int i = 0; i < length; i++)
				{
					symbolTable->lookup(allNChars[i], found);
				}
				QueryPerformanceCounter(&endTime);

				stringstream timeString;
				timeString << endTime.QuadPart - startTime.QuadPart;
				Logger::WriteMessage(timeString.str().c_str());

				delete symbolTable;
			}
			delete[] allNChars;
		}

		TEST_METHOD(timeSomeRandomInsert)
		{
			const int N_CHARS = 2;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);
			srand(2342523);

			random_shuffle(allNChars, allNChars + length);

			LARGE_INTEGER startTime;
			LARGE_INTEGER endTime;
			const int N_TYPES = 5;
			Logger::WriteMessage("timeSomeRandomInsert");
			for (int i = 1; i < N_TYPES - 1; ++i)
			{
				ISymbolTable<int>* symbolTable =
					SymbolTableFactory<int>::create((SymbolTableFactory<int>::TableType) i);

				QueryPerformanceCounter(&startTime);
				for (int i = 0; i < length; i++)
				{
					symbolTable->insert(allNChars[i], i);
				}
				QueryPerformanceCounter(&endTime);

				stringstream timeString;
				timeString << endTime.QuadPart - startTime.QuadPart;
				Logger::WriteMessage(timeString.str().c_str());

				delete symbolTable;
			}
			delete[] allNChars;
		}

		TEST_METHOD(timeSomeAlphaInsert)
		{
			const int N_CHARS = 2;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);
			srand(2342523);

			LARGE_INTEGER startTime;
			LARGE_INTEGER endTime;
			const int N_TYPES = 5;
			Logger::WriteMessage("timeSomeAlphaInsert");
			for (int i = 1; i < N_TYPES - 1; ++i)
			{
				ISymbolTable<int>* symbolTable =
					SymbolTableFactory<int>::create((SymbolTableFactory<int>::TableType) i);

				QueryPerformanceCounter(&startTime);
				for (int i = 0; i < length; i++)
				{
					symbolTable->insert(allNChars[i], i);
				}
				QueryPerformanceCounter(&endTime);

				stringstream timeString;
				timeString << endTime.QuadPart - startTime.QuadPart;
				Logger::WriteMessage(timeString.str().c_str());

				delete symbolTable;
			}
			delete[] allNChars;
		}

		TEST_METHOD(timeSomeAlphaLookup)
		{
			const int N_CHARS = 2;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);
			srand(2342523);

			random_shuffle(allNChars, allNChars + length);

			LARGE_INTEGER startTime;
			LARGE_INTEGER endTime;
			const int N_TYPES = 5;
			Logger::WriteMessage("timeSOmeAlphaLookup");
			for (int i = 1; i < N_TYPES - 1; ++i)
			{
				ISymbolTable<int>* symbolTable =
					SymbolTableFactory<int>::create((SymbolTableFactory<int>::TableType) i);

				for (int i = 0; i < length; i++)
				{
					symbolTable->insert(allNChars[i], i);
				}

				int found;
				QueryPerformanceCounter(&startTime);
				for (int i = 0; i < length; i++)
				{
					symbolTable->lookup(allNChars[i], found);
				}
				QueryPerformanceCounter(&endTime);

				stringstream timeString;
				timeString << endTime.QuadPart - startTime.QuadPart;
				Logger::WriteMessage(timeString.str().c_str());

				delete symbolTable;
			}
			delete[] allNChars;
		}

		TEST_METHOD(timeSomeRandomLookup)
		{
			const int N_CHARS = 2;
			const int BASE = 26;
			int length = 1;
			for (int i = 0; i < N_CHARS; i++)length *= BASE;

			string* allNChars = generateAll(N_CHARS);
			srand(2342523);

			random_shuffle(allNChars, allNChars + length);

			LARGE_INTEGER startTime;
			LARGE_INTEGER endTime;
			const int N_TYPES = 5;
			Logger::WriteMessage("timeSomeRandomLookup");
			for (int i = 1; i < N_TYPES - 1; ++i)
			{
				ISymbolTable<int>* symbolTable =
					SymbolTableFactory<int>::create((SymbolTableFactory<int>::TableType) i);

				for (int i = 0; i < length; i++)
				{
					symbolTable->insert(allNChars[i], i);
				}

				random_shuffle(allNChars, allNChars + length);

				int found;
				QueryPerformanceCounter(&startTime);
				for (int i = 0; i < length; i++)
				{
					symbolTable->lookup(allNChars[i], found);
				}
				QueryPerformanceCounter(&endTime);

				stringstream timeString;
				timeString << endTime.QuadPart - startTime.QuadPart;
				Logger::WriteMessage(timeString.str().c_str());

				delete symbolTable;
			}
			delete[] allNChars;
		}
	};
}